#include "map.h"
#include <stdlib.h>

rect tile_shape(tileset_t const *tileset, int id)
{
    if(!tileset->tiles[id].solid)
        return (rect){ 0 };

    return (rect){
        .l = -fix(0.5),
        .r =  fix(0.5),
        .t = -fix(0.5),
        .b =  fix(0.5),
    };
}

map_cell_t *map_cell(map_t const *m, int x, int y)
{
    if((unsigned)x >= m->width || (unsigned)y >= m->height)
        return NULL;

    return &m->cells[y * m->width + x];
}

bool map_collides(map_t const *m, rect hitbox)
{
    int y_min = ffloor(hitbox.t);
    int y_max = fceil(hitbox.b);
    int x_min = ffloor(hitbox.l);
    int x_max = fceil(hitbox.r);

    /* Collisions against walls and static objects */
    for(int y = y_min; y < y_max; y++)
    for(int x = x_min; x < x_max; x++) {
        map_cell_t *t = map_cell(m, x, y);
        if(!t) continue;

        vec2 c = { fix(x) + fix(0.5), fix(y) + fix(0.5) };
        rect tile_hitbox;

        if(m->tileset->tiles[t->base].solid) {
            tile_hitbox = rect_translate(tile_shape(m->tileset, t->base), c);
            if(rect_collide(hitbox, tile_hitbox))
                return true;
        }
        if(m->tileset->tiles[t->decor].solid) {
            tile_hitbox = rect_translate(tile_shape(m->tileset, t->decor), c);
            if(rect_collide(hitbox, tile_hitbox))
                return true;
        }
    }

    return false;
}
